part of disposable.async;

/// Provides a stand-alone implementation of ensuring clean-up callback
/// which doesn't require sub classing.
class BooleanAsyncDisposable extends BooleanDisposable
    implements AsyncDisposable {
  final Completer _onDispose = new Completer();

  @override
  Future get future => _onDispose.future;

  /// Creates an empty disposable that does nothing when disposed.
  BooleanAsyncDisposable();

  BooleanAsyncDisposable._create(void dispose()) : super.create(dispose);

  /// Creates a disposable object that invokes the specified function
  /// when disposed.
  factory BooleanAsyncDisposable.create(void dispose()) {
    checkNotNull(dispose);

    BooleanAsyncDisposable disposable =
        new BooleanAsyncDisposable._create(dispose);
    disposable.future.catchError((error) {
      //print('Exception raised in Async Callback: ${error}');
      //disposable._handleDisposeError(error);
    });
    return disposable;
  }

  @override
  void dispose() {
    try {
      super.dispose();
      if (!_onDispose.isCompleted) _onDispose.complete();
    } catch (ex) {
      //print('Error in dispose: ${ex}');
      _handleDisposeError(ex);
      throw ex;
    }
  }

  /// Triggers the cleanup callback provided to the disposable
  /// Dispose cannot be called multiple times on the same instance.
  @override
  void disposeWith(dynamic value) {
    try {
      super.dispose();
      if (!_onDispose.isCompleted) _onDispose.complete(value);
    } catch (ex) {
      _handleDisposeError(ex);
      throw ex;
    }
  }

  /// Triggers the cleanup callback provided to the disposable and passes
  /// along the error message.
  /// Dispose cannot be called multiple times on the same instance.
  @override
  void disposeError(dynamic error) {
    try {
      super.dispose();
      if (!_onDispose.isCompleted) _onDispose.completeError(error);
    } catch (ex) {
      _handleDisposeError(ex);
      throw ex;
    }
  }

  void _handleDisposeError(dynamic error) {
    _onDispose.completeError(error);
  }
}
