part of disposable;

/// Represents a disposable resource that can be checked for disposal status.
class BooleanDisposable implements Disposable {
  static Function _noop = () {};
  Function _dispose;

  @override
  bool get isDisposed => _dispose == null;

  /// Creates an empty disposable that does nothing when disposed.
  BooleanDisposable() : _dispose = _noop;

  /// Creates a disposable object that invokes the specified function when disposed.
  BooleanDisposable.create(void dispose()) {
    checkNotNull(dispose);

    _dispose = dispose;
  }

  /// Sets the status to disposed, which can be observer through the
  /// [isDisposed] property.
  @override
  void dispose() {
    final dispose = _dispose;
    _dispose = null;

    if (dispose != null) dispose();
  }
}
