part of disposable.collection;

/// Creates a [List] that automatically disposes its elements when removed.
class DisposableList<E extends Disposable>
    extends MutationTrackingList<DisposableList<E>, E> implements Disposable {
  final CompositeDisposable _disposables = new CompositeDisposable();

  @override
  bool get isDisposed => _disposables.isDisposed;

  DisposableList() : super(new List<E>());

  void dispose() {
    _disposables.dispose();
  }

  @override
  void elementsAdded(int index, Iterable<E> added) {
    for (final element in added) {
      _disposables.add(element);
    }
  }

  @override
  void elementsRemoved(int index, List<E> removed) {
    for (final element in removed) {
      _disposables.remove(element);
    }
  }

  @override
  void elementsReplaced(int index, List<E> removed, Iterable<E> added) {
    for (final element in removed) {
      _disposables.remove(element);
    }

    for (final element in added) {
      _disposables.add(element);
    }
  }
}
