part of disposable;

/// Provides a mechanism for releasing unmanaged resources.
abstract class Disposable {
  /// Creates an empty disposable that does nothing when disposed.
  factory Disposable() => new BooleanDisposable();

  /// Creates a disposable object that invokes the specified
  /// function when disposed.
  factory Disposable.create(void dispose()) =>
      new BooleanDisposable.create(dispose);

  /// Creates a disposable object which cancels the specified
  /// [StreamSubscription] when disposed.
  factory Disposable.from(StreamSubscription subscription) =>
      new BooleanAsyncDisposable.create(subscription.cancel);

  /// Gets a value that indicates whether the object is disposed.
  bool get isDisposed;

  /// Performs application-defined tasks associated with freeing,
  /// releasing, or resetting unmanaged resources.
  void dispose();
}