part of disposable;

/// Represents a disposable whose underlying disposable can
/// be swapped for another disposable.
class MultiAssignmentDisposable implements Disposable {
  bool _disposed = false;
  Disposable _current;

  /// Initializes a new instance of the [MultiAssignmentDisposable] class
  /// with no current underlying disposable.
  MultiAssignmentDisposable();

  @override
  bool get isDisposed => _disposed;

  /// Gets the underlying disposable.
  Disposable get disposable => _current;

  /// Sets the underlying disposable.
  ///
  /// If the [MultipleAssignmentDisposable] has already been disposed,
  /// assignment to this property causes immediate disposal of the
  /// given disposable object.
  void set disposable(Disposable newDisposable) {
    final shouldDispose = _disposed;
    if (!shouldDispose) _current = newDisposable;
    if (shouldDispose && newDisposable != null) newDisposable.dispose();
  }

  @override
  void dispose() {
    var old = null;
    if (!_disposed) {
      _disposed = true;
      old = _current;
      _current = null;
    }
    if (old != null) old.dispose();
  }
}
