part of disposable;

/// Represents a disposable resource whose underlying disposable
/// resource can be replaced by another disposable resource,
/// causing automatic disposal of the previous underlying
/// disposable resource.
class SerialDisposable implements Disposable {
  bool _disposed = false;
  Disposable _current;

  /// Initializes a new instance of the [SerialDisposable] class.
  SerialDisposable();

  @override
  bool get isDisposed => _disposed;

  /// Gets the underlying disposable.
  Disposable get disposable => _current;

  /// Sets the underlying disposable.
  ///
  /// If the [SerialDisposable] has already been disposed, assignment
  /// to this property causes immediate disposal of the given
  /// disposable object. Assigning this property disposes the previous
  /// disposable object.
  void set disposable(Disposable newDisposable) {
    final shouldDispose = _disposed;
    var old = null;
    if (!shouldDispose) {
      old = _current;
      _current = newDisposable;
    }
    if (old != null) old.dispose();
    if (shouldDispose && newDisposable != null) newDisposable.dispose();
  }

  @override
  void dispose() {
    var old = null;
    if (!_disposed) {
      _disposed = true;
      old = _current;
      _current = null;
    }
    if (old != null) old.dispose();
  }
}
