part of disposable;

/// Represents a disposable resource which only allows a single
/// assignment of its underlying disposable resource. If an
/// underlying disposable resource has already been set, future
/// attempts to set the underlying disposable resource will throw
/// an [StateError].
class SingleAssignmentDisposable implements Disposable {
  bool _disposed = false;
  Disposable _current;

  /// Initializes a new instance of the [SingleAssignmentDisposable] class.
  SingleAssignmentDisposable();

  @override
  bool get isDisposed => _disposed;

  /// Gets the underlying disposable.
  Disposable get disposable => _current;

  /// Sets the underlying disposable.
  ///
  /// [StateError] thrown if the SingleAssignmentDisposable has
  /// already been assigned to.
  void set disposable(Disposable newDisposable) {
    if (_current != null) {
      throw new StateError('Disposable has already been assigned');
    }
    final shouldDispose = _disposed;
    if (!shouldDispose) _current = newDisposable;
    if (shouldDispose && newDisposable != null) newDisposable.dispose();
  }

  @override
  void dispose() {
    var old = null;
    if (!_disposed) {
      _disposed = true;
      old = _current;
      _current = null;
    }
    if (old != null) {
      old.dispose();
    }
  }
}
