part of disposable;

/// Provides a method of encapsulating a series of actions which ensure proper
/// disposal of the disposable instance prior to exiting.
void using(Disposable disposable, void usingFunction(Disposable disposable)) {
  var caughtException = null;
  try {
    usingFunction(disposable);
  } catch (ex) {
    caughtException = ex;
  } finally {
    if (caughtException == null) {
      disposable.dispose();
    } else {
      if (disposable is AsyncDisposable) {
        disposable.disposeError(caughtException);
      } else {
        disposable.dispose();
      }
      throw caughtException;
    }
  }
}
