library disposable_test.async_disposable;

import 'package:disposable/async.dart';
import 'package:test/test.dart';

main() {
  group('-async_disposable- should', () {
    test('complete future on dispose', () {
      // Arrange
      bool disposed = false;
      AsyncDisposable disposable = new BooleanAsyncDisposable();
      //disposable.future.then((_) {
      disposable.future.then(expectAsync((_) {
        disposed = true;

        // Assert
        expect(disposable.isDisposed, true);
        expect(disposed, true);
      }));

      // Act
      disposable.dispose();
    });

    test('complete future with value when using disposeWith', () {
      // Arrange
      var value;
      AsyncDisposable disposable = new BooleanAsyncDisposable();
      //disposable.future
      //  .then((v) => value = v);
      disposable.future.then(expectAsync((value) {
        // Assert
        expect(disposable.isDisposed, true);
        expect(value, 'stuff');
      }));

      // Act
      disposable.disposeWith('stuff');
    });

    test(
        'not complete future but should dispose and carry error data in completeError callback when using disposeError',
        () {
      // Arrange
      var futureCalled = false;
      var error;
      AsyncDisposable disposable = new BooleanAsyncDisposable();
      disposable.future.then((_) {
        futureCalled = true;
        expect(false, true);
      }).catchError(expectAsync((error) {
        // Assert
        expect(disposable.isDisposed, true);
        expect(futureCalled, false);
        expect(error, 'error');
      }));

      // Act
      disposable.disposeError('error');
    });

    test('complete with value when disposeWith is called', () {
      // Arrange
      var disposeCalled = false;
      var errorCalled = false;
      var futureCalled = false;
      var error;
      AsyncDisposable disposable =
          new BooleanAsyncDisposable.create(() => disposeCalled = true);
      disposable.future.then((value) {
        expect(value, 10);
        futureCalled = true;
      }, onError: (ex) => error = ex).then(expectAsync((_) {
        // Assert
        expect(disposable.isDisposed, true);
        expect(futureCalled, true);
      }));

      // Act
      try {
        disposable.disposeWith(10);
      } catch (ex) {
        expect(ex, 'dispose error');
      }
    });

    test('complete with error if super throws exception on dispose', () {
      // Arrange
      var errorCalled = false;
      var futureCalled = false;
      var error;
      AsyncDisposable disposable =
          new BooleanAsyncDisposable.create(() => throw 'dispose error');
      disposable.future
          .then((_) => futureCalled = true, onError: (ex) => error = ex)
          .then(expectAsync((_) {
        // Assert
        expect(disposable.isDisposed, true);
        expect(futureCalled, false);
        expect(error, 'dispose error');
      }));

      // Act
      try {
        disposable.dispose();
      } catch (ex) {
        expect(ex, 'dispose error');
      }
    });

    test('complete with error if super throws exception on disposeWith', () {
      // Arrange
      var errorCalled = false;
      var futureCalled = false;
      var error;
      AsyncDisposable disposable =
          new BooleanAsyncDisposable.create(() => throw 'dispose error');
      disposable.future
          .then((_) => futureCalled = true, onError: (ex) => error = ex)
          .then(expectAsync((_) {
        // Assert
        expect(disposable.isDisposed, true);
        expect(futureCalled, false);
        expect(error, 'dispose error');
      }));

      // Act
      try {
        disposable.disposeWith(10);
      } catch (ex) {
        expect(ex, 'dispose error');
      }
    });

    test('complete with error if super throws exception on disposeError', () {
      // Arrange
      var errorCalled = false;
      var futureCalled = false;
      var error;
      AsyncDisposable disposable =
          new BooleanAsyncDisposable.create(() => throw 'dispose error');
      disposable.future
          .then((_) => futureCalled = true, onError: (ex) => error = ex)
          .then(expectAsync((_) {
        // Assert
        expect(disposable.isDisposed, true);
        expect(futureCalled, false);
        expect(error, 'dispose error');
      }));

      // Act
      try {
        disposable.disposeError(10);
      } catch (ex) {
        expect(ex, 'dispose error');
      }
    });
  });
}
