library disposable_test.disposable_list;

import 'package:disposable/collection.dart';
import 'package:disposable/disposable.dart';
import 'package:test/test.dart';

class MyResource implements Disposable {
  final int id;

  bool _isDisposed = false;

  MyResource(this.id);

  void dispose() {
    _isDisposed = true;
  }

  bool get isDisposed => _isDisposed;
}

main() {
  group('DisposableList tests', () {
    DisposableList<MyResource> list;
    MyResource r1;
    MyResource r2;
    MyResource r3;
    MyResource r4;

    setUp(() {
      r1 = new MyResource(1);
      r2 = new MyResource(2);
      r3 = new MyResource(3);
      r4 = new MyResource(4);

      list = new DisposableList<MyResource>()
        ..addAll([r1, r2, r3, r4]);
    });

    test('disposing the list disposes all the elements in the list', () {
      list.dispose();
      expect(list.isDisposed, isTrue);

      expect(r1.isDisposed, isTrue);
      expect(r2.isDisposed, isTrue);
      expect(r3.isDisposed, isTrue);
      expect(r4.isDisposed, isTrue);
    });

    test('removing an element from the list disposes the element', () {
      expect(r1.isDisposed, isFalse);

      list.remove(r1);
      expect(r1.isDisposed, isTrue);
    });

    test('replacing an element disposes the replaced element', () {
      expect(r1.isDisposed, isFalse);

      list[0] = new MyResource(0);
      expect(r1.isDisposed, isTrue);
    });
  });
}
