library disposable_test.disposable;

import 'dart:async';

import 'package:disposable/disposable.dart';
import 'package:test/test.dart';

main() {
  group('-disposable-', () {
    test(
        'not throw exception on dispose when no callback is provided, aka use noop instead of null',
        () {
      // Arrange
      Disposable disposable = new Disposable();
      expect(() => disposable.dispose(), returnsNormally);
    });

    test('flip isDisposed when disposed', () {
      // Arrange
      Disposable disposable = new Disposable();

      // Act
      expect(disposable.isDisposed, false);
      disposable.dispose();

      // Assert
      expect(disposable.isDisposed, true);
    });

    test('trigger callback when disposed', () {
      // Arrange
      bool disposed = false;
      Disposable disposable = new Disposable.create(() {
        disposed = true;
      });

      // Act
      disposable.dispose();

      // Assert
      expect(disposed, true);
    });

    test('cancel StreamSubscription when disposed', () {
      final streamController = new StreamController<int>(sync: true);
      final stream = streamController.stream;
      final subscription = stream.listen(expectAsync((i) {
        expect(i, equals(0));
      }));
      Disposable disposable = new Disposable.from(subscription);

      streamController.add(0);
      disposable.dispose();

      streamController.add(1);
    });

    test('only trigger dispose method at most once', () {
      // Arrange
      int disposedCount = 0;
      var callback = () {
        disposedCount++;
      };
      Disposable disposable = new Disposable.create(callback);

      // Act
      disposable.dispose();
      disposable.dispose();
      disposable.dispose();

      // Assert
      expect(disposedCount, 1);
    });

    test('not trap error if unhandled exception is raised in callback', () {
      // Arrange
      var callback = () => throw 'error';
      Disposable disposable = new Disposable.create(callback);

      expect(() => disposable.dispose(), throwsA(new isInstanceOf<String>()));
    });
  });
}
