library disposable_test.ref_count_disposable;

import 'package:disposable/disposable.dart';
import 'package:test/test.dart';

main() {
  group('RefCountDisposable tests', () {
    test('RefCountDisposable_Ctor_Null', () {
      expect(() => new RefCountDisposable(null), throwsArgumentError);
    });

    test('RefCountDisposable_SingleReference', () {
      final d = new Disposable();
      final r = new RefCountDisposable(d);
      expect(d.isDisposed, isFalse);
      r.dispose();
      expect(d.isDisposed, isTrue);
      r.dispose();
      expect(d.isDisposed, isTrue);
    });

    test('RefCountDisposable_RefCounting', () {
      final d = new Disposable();
      final r = new RefCountDisposable(d);
      expect(d.isDisposed, isFalse);

      final d1 = r.getDisposable();
      final d2 = r.getDisposable();
      expect(d.isDisposed, isFalse);

      d1.dispose();
      expect(d.isDisposed, isFalse);

      d2.dispose();
      expect(d.isDisposed, isFalse);

      r.dispose();
      expect(d.isDisposed, isTrue);
      expect(r.isDisposed, isTrue);

      final d3 = r.getDisposable();
      d3.dispose();
    });

    test('RefCountDisposable_PrimaryDisposesFirst', () {
      final d = new Disposable();
      final r = new RefCountDisposable(d);
      expect(d.isDisposed, isFalse);

      final d1 = r.getDisposable();
      final d2 = r.getDisposable();
      expect(d.isDisposed, isFalse);

      d1.dispose();
      expect(d.isDisposed, isFalse);

      r.dispose();
      expect(d.isDisposed, isFalse);

      d2.dispose();
      expect(d.isDisposed, isTrue);
    });
  });
}
