library disposable_test.serial_disposable;

import 'package:disposable/disposable.dart';
import 'package:test/test.dart';

main() {
  group('SerialDisposable tests', () {
    test('SerialDisposable_Ctor_Prop', () {
      final m = new SerialDisposable();
      expect(m.disposable, isNull);
    });

    test('SerialDisposable_ReplaceBeforeDispose', () {
      var disp1 = false;
      var disp2 = false;

      final m = new SerialDisposable();
      final d1 = new Disposable.create(() {
        disp1 = true;
      });
      m.disposable = d1;
      expect(d1, same(m.disposable));
      expect(disp1, isFalse);

      final d2 = new Disposable.create(() {
        disp2 = true;
      });
      m.disposable = d2;
      expect(d2, same(m.disposable));
      expect(disp1, isTrue);
      expect(disp2, isFalse);
    });

    test('SerialDisposable_ReplaceAfterDispose', () {
      var disp1 = false;
      var disp2 = false;

      final m = new SerialDisposable();
      m.dispose();
      expect(m.isDisposed, isTrue);

      final d1 = new Disposable.create(() {
        disp1 = true;
      });
      m.disposable = d1;
      expect(m.disposable, isNull);
      expect(disp1, isTrue);

      final d2 = new Disposable.create(() {
        disp2 = true;
      });
      m.disposable = d2;
      expect(m.disposable, isNull);
      expect(disp2, isTrue);
    });

    test('SerialDisposable_Dispose', () {
      var disp = false;

      final m = new SerialDisposable();
      final d = new Disposable.create(() {
        disp = true;
      });
      m.disposable = d;
      expect(d, same(m.disposable));
      expect(disp, isFalse);

      m.dispose();
      expect(m.isDisposed, isTrue);
      expect(disp, isTrue);
    });
  });
}
