library disposable_test.using;

import 'package:disposable/async.dart';
import 'package:disposable/disposable.dart';
import 'package:test/test.dart';

main() {
  group('-using- should', () {
    test('trigger dispose when exiting using statement', () {
      // Arrange
      bool disposed = false;
      Disposable disposable =
          new Disposable.create(() => disposed = true);

      // Act
      using(disposable, (Disposable disposable) {
        expect(disposed, false);
      });

      // Assert
      expect(disposed, true);
    });

    test('still trigger dispose if wrapped block throws exception', () {
      // Arrange
      bool disposed = false;
      Disposable disposable =
          new Disposable.create(() => disposed = true);
      // Act
      try {
        using(disposable, (Disposable disposable) {
          throw 'error';
        });
      } catch (ex) {}

      // Assert
      expect(disposed, true);
    });

    test('rethrow caught exception after disposing', () {
      // Arrange
      bool disposed = false;
      bool caughtError = false;
      var error = null;
      Disposable disposable =
          new Disposable.create(() => disposed = true);

      // Act
      try {
        using(disposable, (Disposable disposable) {
          throw 'error';
        });
      } catch (ex) {
        caughtError = true;
        error = ex;
      }

      // Assert
      expect(disposed, true);
      expect(caughtError, true,
          reason: 'should have been re-thrown by using block');
      expect(error, 'error');
    });

    test(
        'trigger Future.completeError if disposable is IAsyncDisposable and exception is thrown in wrapped block',
        () {
      // Arrange
      bool disposed = false;
      bool futureCalled = false;
      var error = null;
      AsyncDisposable disposable =
          new BooleanAsyncDisposable.create(() => disposed = true);

      var future = disposable.future
          .then((_) => futureCalled = true)
          .catchError((ex) => error = ex)
          .then(expectAsync((_) {
        expect(futureCalled, false);
        expect(error, 'error',
            reason:
                'if the future error handler was triggered this will not be null');
      }));

      // Act
      try {
        using(disposable, (AsyncDisposable disposable) {
          throw 'error';
        });
      } catch (ex) {}

      expect(disposed, true);
    });
  });
}
